#ifndef __RESOURCE_xfdesktop_monitor_chooser_ui_H__
#define __RESOURCE_xfdesktop_monitor_chooser_ui_H__

#include <gio/gio.h>

extern GResource *xfdesktop_monitor_chooser_ui_get_resource (void);

extern void xfdesktop_monitor_chooser_ui_register_resource (void);
extern void xfdesktop_monitor_chooser_ui_unregister_resource (void);

#endif
